@@ -34,6 +34,7 @@
 #include <libsoup/soup.h>
 #include <string.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 #include "gssdp-resource-browser.h"
 #include "gssdp-client-private.h"
@@ -63,6 +64,7 @@ struct _GSSDPResourceBrowserPrivate {
                         
         GSource     *timeout_src;
         guint        num_discovery;
+        guint        version;
 };
 
 enum {
@@ -124,7 +126,7 @@ gssdp_resource_browser_init (GSSDPResourceBrowser *resource_browser)
         resource_browser->priv->resources =
                 g_hash_table_new_full (g_str_hash,
                                        g_str_equal,
-                                       NULL,
+                                       g_free,
                                        resource_free);
 }
 
@@ -257,7 +259,7 @@ gssdp_resource_browser_class_init (GSSDPResourceBrowserClass *klass)
         g_type_class_add_private (klass, sizeof (GSSDPResourceBrowserPrivate));
 
         /**
-         * GSSDPResourceBrowser:client
+         * GSSDPResourceBrowser:client:
          *
          * The #GSSDPClient to use.
          **/
@@ -274,7 +276,7 @@ gssdp_resource_browser_class_init (GSSDPResourceBrowserClass *klass)
                           G_PARAM_STATIC_BLURB));
 
         /**
-         * GSSDPResourceBrowser:target
+         * GSSDPResourceBrowser:target:
          *
          * The discovery target.
          **/
@@ -291,7 +293,7 @@ gssdp_resource_browser_class_init (GSSDPResourceBrowserClass *klass)
                           G_PARAM_STATIC_BLURB));
 
         /**
-         * GSSDPResourceBrowser:mx
+         * GSSDPResourceBrowser:mx:
          *
          * The maximum number of seconds in which to request other parties
          * to respond.
@@ -312,7 +314,7 @@ gssdp_resource_browser_class_init (GSSDPResourceBrowserClass *klass)
                           G_PARAM_STATIC_BLURB));
 
         /**
-         * GSSDPResourceBrowser:active
+         * GSSDPResourceBrowser:active:
          *
          * Whether this browser is active or not.
          **/
@@ -329,7 +331,7 @@ gssdp_resource_browser_class_init (GSSDPResourceBrowserClass *klass)
                           G_PARAM_STATIC_BLURB));
 
         /**
-         * GSSDPResourceBrowser::resource-available
+         * GSSDPResourceBrowser::resource-available:
          * @resource_browser: The #GSSDPResourceBrowser that received the
          * signal
          * @usn: The USN of the discovered resource
@@ -353,7 +355,7 @@ gssdp_resource_browser_class_init (GSSDPResourceBrowserClass *klass)
                               G_TYPE_POINTER);
 
         /**
-         * GSSDPResourceBrowser::resource-unavailable
+         * GSSDPResourceBrowser::resource-unavailable:
          * @resource_browser: The #GSSDPResourceBrowser that received the
          * signal
          * @usn: The USN of the resource
@@ -375,7 +377,7 @@ gssdp_resource_browser_class_init (GSSDPResourceBrowserClass *klass)
 }
 
 /**
- * gssdp_resource_browser_new
+ * gssdp_resource_browser_new:
  * @client: The #GSSDPClient to associate with
  *
  * Return value: A new #GSSDPResourceBrowser object.
@@ -413,7 +415,7 @@ gssdp_resource_browser_set_client (GSSDPResourceBrowser *resource_browser,
 }
 
 /**
- * gssdp_resource_browser_get_client
+ * gssdp_resource_browser_get_client:
  * @resource_browser: A #GSSDPResourceBrowser
  *
  * Returns: (transfer none): The #GSSDPClient @resource_browser is associated with.
@@ -428,7 +430,7 @@ gssdp_resource_browser_get_client (GSSDPResourceBrowser *resource_browser)
 }
 
 /**
- * gssdp_resource_browser_set_target
+ * gssdp_resource_browser_set_target:
  * @resource_browser: A #GSSDPResourceBrowser
  * @target: The browser target
  *
@@ -453,7 +455,7 @@ gssdp_resource_browser_set_target (GSSDPResourceBrowser *resource_browser,
         if (resource_browser->priv->target_regex)
                 g_regex_unref (resource_browser->priv->target_regex);
 
-        version_pattern = "[0-9]+";
+        version_pattern = "([0-9]+)";
         /* Make sure we have enough room for version pattern */
         pattern = g_strndup (target,
                              strlen (target) + strlen (version_pattern));
@@ -464,8 +466,9 @@ gssdp_resource_browser_set_target (GSSDPResourceBrowser *resource_browser,
              version != g_strstr_len (pattern, -1, ":")) &&
             g_regex_match_simple (version_pattern,
                                   version + 1,
-                                  G_REGEX_MATCH_ANCHORED,
-                                  0)) {
+                                  G_REGEX_ANCHORED,
+                                  G_REGEX_MATCH_ANCHORED)) {
+                resource_browser->priv->version = atoi (version + 1);
                 strcpy (version + 1, version_pattern);
         }
 
@@ -487,7 +490,7 @@ gssdp_resource_browser_set_target (GSSDPResourceBrowser *resource_browser,
 }
 
 /**
- * gssdp_resource_browser_get_target
+ * gssdp_resource_browser_get_target:
  * @resource_browser: A #GSSDPResourceBrowser
  *
  * Return value: The browser target.
@@ -502,7 +505,7 @@ gssdp_resource_browser_get_target (GSSDPResourceBrowser *resource_browser)
 }
 
 /**
- * gssdp_resource_browser_set_mx
+ * gssdp_resource_browser_set_mx:
  * @resource_browser: A #GSSDPResourceBrowser
  * @mx: The to be used MX value
  *
@@ -523,7 +526,7 @@ gssdp_resource_browser_set_mx (GSSDPResourceBrowser *resource_browser,
 }
 
 /**
- * gssdp_resource_browser_get_mx
+ * gssdp_resource_browser_get_mx:
  * @resource_browser: A #GSSDPResourceBrowser
  *
  * Return value: The used MX value.
@@ -537,7 +540,7 @@ gssdp_resource_browser_get_mx (GSSDPResourceBrowser *resource_browser)
 }
 
 /**
- * gssdp_resource_browser_set_active
+ * gssdp_resource_browser_set_active:
  * @resource_browser: A #GSSDPResourceBrowser
  * @active: TRUE to activate @resource_browser
  *
@@ -566,7 +569,7 @@ gssdp_resource_browser_set_active (GSSDPResourceBrowser *resource_browser,
 }
 
 /**
- * gssdp_resource_browser_get_active
+ * gssdp_resource_browser_get_active:
  * @resource_browser: A #GSSDPResourceBrowser
  *
  * Return value: TRUE if @resource_browser is active.
@@ -588,6 +591,7 @@ resource_expire (gpointer user_data)
         GSSDPResourceBrowser *resource_browser;
         Resource *resource;
         char *usn;
+        char *canonical_usn;
 
         resource = user_data;
         resource_browser = resource->resource_browser;
@@ -598,13 +602,24 @@ resource_expire (gpointer user_data)
         usn = resource->usn;
         resource->usn = NULL;
 
-        g_hash_table_remove (resource->resource_browser->priv->resources, usn);
+        if (resource_browser->priv->version > 0) {
+                char *version;
+
+                version = g_strrstr (usn, ":");
+                canonical_usn = g_strndup (usn, version - usn);
+        } else {
+                canonical_usn = g_strdup (usn);
+        }
+
+        g_hash_table_remove (resource->resource_browser->priv->resources,
+                             canonical_usn);
 
         g_signal_emit (resource_browser,
                        signals[RESOURCE_UNAVAILABLE],
                        0,
                        usn);
         g_free (usn);
+        g_free (canonical_usn);
 
         return FALSE;
 }
@@ -619,13 +634,24 @@ resource_available (GSSDPResourceBrowser *resource_browser,
         gboolean was_cached;
         guint timeout;
         GList *locations;
+        char *canonical_usn;
 
         usn = soup_message_headers_get_one (headers, "USN");
         if (!usn)
                 return; /* No USN specified */
 
+        if (resource_browser->priv->version > 0) {
+                char *version;
+
+                version = g_strrstr (usn, ":");
+                canonical_usn = g_strndup (usn, version - usn);
+        } else {
+                canonical_usn = g_strdup (usn);
+        }
+
         /* Get from cache, if possible */
-        resource = g_hash_table_lookup (resource_browser->priv->resources, usn);
+        resource = g_hash_table_lookup (resource_browser->priv->resources,
+                                        canonical_usn);
         if (resource) {
                 /* Remove old timeout */
                 g_source_destroy (resource->timeout_src);
@@ -639,12 +665,18 @@ resource_available (GSSDPResourceBrowser *resource_browser,
                 resource->usn              = g_strdup (usn);
                 
                 g_hash_table_insert (resource_browser->priv->resources,
-                                     resource->usn,
+                                     canonical_usn,
                                      resource);
                 
                 was_cached = FALSE;
+
+                /* hash-table takes ownership of this */
+                canonical_usn = NULL;
         }
 
+        if (canonical_usn != NULL)
+                g_free (canonical_usn);
+
         /* Calculate new timeout */
         header = soup_message_headers_get_one (headers, "Cache-Control");
         if (header) {
@@ -774,33 +806,77 @@ resource_unavailable (GSSDPResourceBrowser *resource_browser,
                       SoupMessageHeaders   *headers)
 {
         const char *usn;
+        char *canonical_usn;
 
         usn = soup_message_headers_get_one (headers, "USN");
         if (!usn)
                 return; /* No USN specified */
 
-        /* Only process if we were cached */
-        if (!g_hash_table_lookup (resource_browser->priv->resources, usn))
-                return;
+        if (resource_browser->priv->version > 0) {
+                char *version;
 
-        g_hash_table_remove (resource_browser->priv->resources, usn);
+                version = g_strrstr (usn, ":");
+                canonical_usn = g_strndup (usn, version - usn);
+        } else {
+                canonical_usn = g_strdup (usn);
+        }
+
+        /* Only process if we were cached */
+        if (!g_hash_table_lookup (resource_browser->priv->resources,
+                                  canonical_usn))
+                goto out;
+
+        g_hash_table_remove (resource_browser->priv->resources,
+                             canonical_usn);
 
         g_signal_emit (resource_browser,
                        signals[RESOURCE_UNAVAILABLE],
                        0,
                        usn);
+
+out:
+        g_free (canonical_usn);
 }
 
 static gboolean
 check_target_compat (GSSDPResourceBrowser *resource_browser,
                      const char           *st)
 {
-        return strcmp (resource_browser->priv->target,
-                       GSSDP_ALL_RESOURCES) == 0 ||
-               g_regex_match (resource_browser->priv->target_regex,
-                              st,
-                              0,
-                              NULL);
+        GMatchInfo *info;
+        int         version;
+        char       *tmp;
+
+        if (strcmp (resource_browser->priv->target,
+                    GSSDP_ALL_RESOURCES) == 0)
+                return TRUE;
+
+        if (!g_regex_match (resource_browser->priv->target_regex,
+                            st,
+                            0,
+                            &info)) {
+                g_match_info_free (info);
+
+                return FALSE;
+        }
+
+        /* If there was no version to match, we're done */
+        if (resource_browser->priv->version == 0) {
+                g_match_info_free (info);
+
+                return TRUE;
+        }
+
+        if (g_match_info_get_match_count (info) != 2) {
+                g_match_info_free (info);
+
+                return FALSE;
+        }
+
+        version = atoi ((tmp = g_match_info_fetch (info, 1)));
+        g_free (tmp);
+        g_match_info_free (info);
+
+        return version >= resource_browser->priv->version;
 }
 
 static void
